#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

typedef long int INTEGER;
typedef long double REAL;
typedef int BOOL;

enum { FALSE, TRUE };

enum FILE_ID { INPF, OUTF, ANSF };

enum CHECK_RESULT {
  CHK_OK                 = 0,
  CHK_WRONG_ANSWER       = 1,
  CHK_PRESENTATION_ERROR = 2,
  CHK_INTERNAL_ERROR     = 3,
  CHK_ERR_NO_MEMORY      ,
  CHK_ERR_NO_INPUT_TXT   ,
  CHK_ERR_NO_OUTPUT_TXT  ,
  CHK_ERR_NO_ANSWER      ,
};

enum {
  DEFAULT_STRING_LENGTH = 256,
  MAX_STRING_LENGTH     = 10000
};

typedef char STRING[DEFAULT_STRING_LENGTH];
typedef char LONG_STRING[MAX_STRING_LENGTH];

void msg(char *format, ...);
void init_check(int argc, char **argv);
void finish_check(enum CHECK_RESULT res);
void check_mem(INTEGER memory_required);
void set_string_length(int n);

BOOL check_eof(enum FILE_ID f);
BOOL seek_eof(enum FILE_ID f);
void require_eof(enum FILE_ID f);

INTEGER read_integer(enum FILE_ID f);
REAL read_real(enum FILE_ID f);
char read_char(enum FILE_ID f);
char *read_string(enum FILE_ID f);
char *read_large(enum FILE_ID f);

void skip_spaces(enum FILE_ID f);



FILE *files[3];
const char *file_texts[3] = {"input", "output", "answers"};

int string_length;
BOOL check_init = FALSE;

void msg(char *format, ...)
{
  va_list L;
  printf("CHECK: ");
  va_start(L, format);
  vprintf(format, L);
  printf("\n");
  va_end(L);
}

void CHKI()
{
  if (!check_init) {
    msg("!CHKI!");
    finish_check(CHK_INTERNAL_ERROR);
  }
}

char read_char(enum FILE_ID f) {
  char ch;
  if (check_eof(f)) {
    msg("Unexpected end of %s file", file_texts[f]);
    finish_check(CHK_PRESENTATION_ERROR);
  }
  return fgetc(files[f]);
}


void finish_check(enum CHECK_RESULT res)
{
  if (res < CHK_INTERNAL_ERROR) {
//    fclose(files[INPF]); mike
//    fclose(files[OUTF]);
//    fclose(files[ANSF]);

    if(files[INPF]) fclose(files[INPF]);
    if(files[OUTF]) fclose(files[OUTF]);
    if(files[ANSF]) fclose(files[ANSF]);
  }
  //else res = CHK_PRESENTATION_ERROR;
  exit((int)res);
}

FILE *safe_fopen(const char *fname, enum CHECK_RESULT err_res)
{
  FILE *fres;
  fres = fopen(fname, "rb");
  if (!fres) {
    msg("Error opening file '%s'", fname);
    finish_check(err_res);
  }
  return fres;
}

// void open_files(const char *in_name, const char *out_name, const char *ans_name) - mike
void open_files(const char *ans_name, const char *in_name, const char *out_name)
{
  files[INPF] = safe_fopen(in_name, CHK_ERR_NO_INPUT_TXT);
  files[OUTF] = safe_fopen(out_name, CHK_PRESENTATION_ERROR /*CHK_ERR_NO_OUTPUT_TXT*/);
  files[ANSF] = safe_fopen(ans_name, CHK_ERR_NO_ANSWER);
}

void set_string_length(int n)
{
  if (n > MAX_STRING_LENGTH) {
     msg("set_string_length(%d): MAX_STRING_LENGTH exceeded", n);
     string_length = MAX_STRING_LENGTH;
  }
  if (n < 0) {
     msg("set_string_length(%d): < 0, set to deafult", n);
     string_length = DEFAULT_STRING_LENGTH;
  }
  string_length = n;
}

void init_check(int argc, char **argv)
{
  if (argc != 4) {
    msg("Wrong nuber of arguments: %d", argc - 1);
    finish_check(CHK_INTERNAL_ERROR);
  }
  open_files(argv[1], argv[3], argv[2]);

  set_string_length(DEFAULT_STRING_LENGTH);
  check_init = TRUE;
}

void check_mem(INTEGER memory_required)
{ return;
//  #ifndef __GNUC__ - mike
  #ifdef __GNUC__
  if (coreleft() < memory_required) {
    msg("Not enough memory");
    finish_check(CHK_ERR_NO_MEMORY);
  }
  #endif
}

BOOL check_eof(enum FILE_ID f)
{
  CHKI();
  return feof(files[f]);
}

BOOL seek_eof(enum FILE_ID f)
{
  skip_spaces(f);
  return check_eof(f);
}

void require_eof(enum FILE_ID f)
{
  if (!seek_eof(f)) {
    msg("Extra characters in %s file", file_texts[f]);
    finish_check(CHK_PRESENTATION_ERROR);
  }
}

void skip_spaces(enum FILE_ID f) {
  int ch;
  while ((ch = fgetc(files[f])) != EOF && isspace(ch));
    ;
  if (!check_eof(f)) ungetc(ch, files[f]);
}

char word_buf[MAX_STRING_LENGTH];

char* read_word(enum FILE_ID f)
{
  char *p;
  int ch;

  skip_spaces(f);
  for (p = word_buf; (ch = fgetc(files[f])) != EOF && !isspace(ch); p++) {
    *p = ch;
    if (p - word_buf >= string_length) {
      msg("Word is too long!");
      finish_check(CHK_PRESENTATION_ERROR);
    }
  }
  if (p==word_buf) {
    msg("Word not found!");
    finish_check(CHK_PRESENTATION_ERROR);
  }
  skip_spaces(f); // scan for eof!
  *p = 0;
  return word_buf;
}

void PE(char *type, enum FILE_ID f)
{
  msg("Found '%s' instead of %s in %s", word_buf, type, file_texts[f]);
  finish_check(CHK_PRESENTATION_ERROR);
}

INTEGER read_integer(enum FILE_ID f)
{
  INTEGER r;
  char *end;

  r = strtol(read_word(f), &end, 10);
  if (*end != 0 || end == word_buf) PE("INTEGER", f);
  return r;
}

REAL read_real(enum FILE_ID f)
{
  REAL r;
  char *end;

//  r = strtold(read_word(f), &end); - mike
  r = strtod(read_word(f), &end);
  if (*end != 0 || end == word_buf) PE("REAL", f);
  return r;
}

char *read_string(enum FILE_ID f)
{
  return read_word(f);
}

char *read_large(enum FILE_ID f)
{
  int sgn;
  char *p, *s;

  read_word(f);
  p = word_buf;
  sgn = (*p == '-') ? -1 : 1;
  if (*p == '+' || *p == '-') p++;

  while (*p == '0' && p[1]) p++;
  if (!*p) PE("LARGE", f);
  if (!p[1]) sgn = 0;
  
  if (sgn < 0) *word_buf = '-';
  p = strcpy(sgn < 0 ? word_buf + 1: word_buf, p);  

  while(*p) if (!isdigit(*p++)) PE("LARGE", f);
        
  return word_buf;
}


int main(int argc, char **argv)
{
    int s[1001];
    int N, M, K, i, j, t;

    check_mem(500000L);
    init_check(argc, argv);

    N = read_integer(INPF);
    M = read_integer(INPF);
    K = read_integer(OUTF);

    if (K != read_integer(ANSF)) 
        finish_check(CHK_WRONG_ANSWER);

    if (!K)
        finish_check(CHK_OK);


    for (i = 1; i <= N; i++) 
        s[i] = 1;

    for (i = 1; i <= K; i++)
        for (j = 1; j <= M; j++) {
            t = read_integer(OUTF);
            if (t < 1 || t > N) {
                msg("wrong glass");
                finish_check(CHK_WRONG_ANSWER);
            }
            s[t] = 1 - s[t];
        }
      
    for (i = 1; i <= N; i++)
    {
        if (s[i]) {
            msg("incomplete");
            finish_check(CHK_WRONG_ANSWER);
        }
    }
  
    require_eof(OUTF);
    finish_check(CHK_OK);
    return 0;
}